//
//  TransRunner.m
//  Uniterm
//
//  Created by John Schember on 12/29/15.
//  Copyright © 2015 Monetra Technologies, LLC All rights reserved.
//

#import <Foundation/Foundation.h>
#include <Uniterm/Uniterm.h>

#import "TransRunner.h"

@implementation TransRunner

@synthesize delegate;

NSString *_username;
NSString *_password;
NSString *_uid;
BOOL _running;

- (void)runCancelRunner {
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^ {
        NSDictionary *req;

        @synchronized(self) {
            if (_uid == nil)
                return;
            
            req = @{
                    @"u_action": @"cancel",
                    @"username": _username,
                    @"password": _password,
                    @"u_id": _uid
                    };
        }
        
        uniterm_run_trans(req);
    });
}

- (void)runStatusRunner {
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^ {
        NSDictionary *req;

        @synchronized(self) {
            if (_uid == nil) {
                [delegate updateStatus:@"Working..." allowCancel:NO];
                return;
            }
                
            req = @{
                    @"u_action": @"status",
                    @"username": _username,
                    @"password": _password,
                    @"u_id": _uid
                    };
        }
        
        while (YES) {
            @synchronized(self) {
                if (_uid == nil) {
                    return;
                }
            }
            
            NSDictionary *rsp = uniterm_run_trans(req);
            
            NSString *code      = [rsp objectForKey:@"u_errorcode"];
            NSString *verbiage  = [rsp objectForKey:@"verbiage"];
            BOOL      canCancel = NO;
            
            if (code == nil || ![[code lowercaseString] isEqualToString:@"success"] || verbiage == nil) {
                verbiage = @"Working...";
            }
            
            if ([rsp objectForKey:@"u_cancelable"] && [[rsp objectForKey:@"u_cancelable"] isEqualToString:@"yes"]) {
                canCancel = YES;
            }
            
            [delegate updateStatus:verbiage allowCancel:canCancel];
            [NSThread sleepForTimeInterval:0.1f];
        }
    });
}

- (void)runTransRunner:(NSDictionary *)request {
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^ {
        NSMutableDictionary *req;
        NSDictionary        *ursp;
        NSMutableDictionary *rsp;

        // Generate the request object. We many need to modify it.
        req = [[NSMutableDictionary alloc] initWithDictionary:request copyItems:YES];
        
        // We want to have status updates for txnrequests.
        if ([request objectForKey:@"u_action"] &&
            [[[request objectForKey:@"u_action"] lowercaseString] isEqualToString:@"txnrequest"])
        {
            @synchronized(self) {
                _username = [request objectForKey:@"username"];
                _password = [request objectForKey:@"password"];
                _uid = [request objectForKey:@"u_id"];
                // If no u_id was set we need to make on up for status.
                if (_uid == nil) {
                    _uid = [NSString stringWithFormat:@"%lu", (unsigned long)arc4random()];
                    [req setObject:_uid forKey:@"u_id"];
                }
            }
        }
        [self runStatusRunner];
        
        ursp = uniterm_run_trans(req);

        // Kill stored data so we can stop the status runner if it's running.
        // Also prevents cancel (can't cancel at this point).
        @synchronized(self) {
            _username = nil;
            _password = nil;
            _uid = nil;
            _running = NO;
        }
        
        // Ensure there is a response (could be nil due to fatal failure).
        if (ursp == nil || [ursp count] == 0) {
            rsp = [NSMutableDictionary dictionary];
            [rsp setObject:@"DENY" forKey:@"u_errorcode"];
            [rsp setObject:@"DENY" forKey:@"code"];
            [rsp setObject:@"Failure" forKey:@"verbiage"];
        } else {
            rsp = [[NSMutableDictionary alloc] initWithDictionary:ursp copyItems:YES];
        }
        
        [delegate sendResult:rsp];
    });
}

+ (id)transRunner
{
    return [[TransRunner alloc] init];
}

- (id)init
{
    self = [super init];
    if (self == nil)
        return nil;
    
    return self;
}

- (void)runTransRequest:(NSDictionary *)req
{
    @synchronized(self) {
        if (_running)
            return;

        _running = YES;
        [delegate updateStatus:@"Uniterm" allowCancel:NO];
        [self runTransRunner:req];
    }
}

- (void)cancelRunningTran
{
    [self runCancelRunner];
}

@end
